package com.watabou.newquay;

import java.util.regex.Pattern;

import com.watabou.newquay.utils.Point;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;

public class BitmapTextMultiline extends BitmapText {

	public int maxWidth = Integer.MAX_VALUE;
	
	protected static final Pattern PARAGRAPH= Pattern.compile( "\n" );
	protected static final Pattern WORD		= Pattern.compile( "\\s+" );
	
	protected int spaceSize;
	
	public BitmapTextMultiline( Font font ) {
		this( "", font ); 
	}
	
	public BitmapTextMultiline( String text, Font font ) {
		super( text, font );
		spaceSize = font.get( ' ' ).width();
	}
	
	@Override
	protected void draw( Canvas canvas, int x, int y ) {
		
		//        
		SymbolWriter writer = new SymbolWriter();
		
		//      
		Point metrics = new Point();
		
		//    
		Rect dstRect = new Rect();
		
		//  .     
		//     
		String paragraphs[] = PARAGRAPH.split( text );
		
		for (int i=0; i < paragraphs.length; i++) {
			
			//      
			String[] words = WORD.split( paragraphs[i] );
			
			for (int j=0; j < words.length; j++) {
				
				String word = words[j];
				if (word.length() == 0) {
					//    , , 
					//  ,   
					continue;
				}
				
				//   ,      
				getWordMetrics( word, metrics );	
				writer.addSymbol( metrics.x, metrics.y );
				
				int length = word.length();
				int shift = 0;	//       
				
				for (int k=0; k < length; k++) {
					
					Rect srcRect = font.get( word.charAt( k ) );
					if (srcRect == null) {
						//  ,    
						continue;
					}
					
					dstRect.set( srcRect );
					dstRect.offsetTo( x + writer.x + shift, y + writer.y );
					canvas.drawBitmap( font.bitmap, srcRect, dstRect, paint );
					
					shift += srcRect.width() + font.tracking;
				}
				
				writer.addSpace( spaceSize );
			}
			
			writer.newLine();
		}
	}
	
	private void getWordMetrics( String word, Point metrics ) {
		
		int w = 0;
		int h = 0;
		
		int length = word.length();
		for (int i=0; i < length; i++) {
			
			Rect rect = font.get( word.charAt( i ) );
			if (rect == null) {
				continue;
			}
			
			w += rect.width() + (w > 0 ? font.tracking : 0);
			h = Math.max( h, rect.height() );
		}
		
		metrics.set( w, h );
	}
	
	@Override
	public void measure() {
		
		SymbolWriter writer = new SymbolWriter();
		
		Point metrics = new Point();
		
		String paragraphs[] = PARAGRAPH.split( text );
		
		for (int i=0; i < paragraphs.length; i++) {
			
			String[] words = WORD.split( paragraphs[i] );
			
			for (int j=0; j < words.length; j++) {
				
				String word = words[j];
				if (word.length() == 0) {
					continue;
				}
				
				getWordMetrics( word, metrics );	
				writer.addSymbol( metrics.x, metrics.y );
				writer.addSpace( spaceSize );
			}
			
			writer.newLine();
		}
		writer.newLine();
		
		width = writer.width;
		height = writer.height;
	}
	
	public static Image createImage( String text, Font font, int maxWidth, int color ) {
		
		BitmapTextMultiline txt = new BitmapTextMultiline( text, font );
		txt.maxWidth = maxWidth;
		txt.setColor( color );
		txt.measure();
		
		Bitmap bmp = Bitmap.createBitmap( (int)txt.width, (int)txt.height, font.bitmap.getConfig() );
		txt.draw( new Canvas( bmp ), 0, 0 );
		
		return new Image( bmp );
	}
	
	private class SymbolWriter {
		
		public int width = 0;
		public int height = 0;
		
		public int lineWidth = 0;
		public int lineHeight = 0;
		
		public int x = 0;
		public int y = 0;
		
		public void addSymbol( int w, int h ) {
			if (lineWidth > 0 && lineWidth + font.tracking + w > maxWidth) {
				newLine( w, h );
			} else {
				
				x = lineWidth;
				
				lineWidth += (lineWidth > 0 ? font.tracking : 0) + w;
				if (h > lineHeight) {
					lineHeight = h;
				}
			}
		}
		
		public void addSpace( int w ) {
			if (lineWidth > 0 && lineWidth + font.tracking + w > maxWidth) {
				newLine( 0, 0 );
			} else {
				
				x = lineWidth;
				lineWidth += (lineWidth > 0 ? font.tracking : 0) + w;
			}
		}
		
		public void newLine() {
			newLine( 0, 0 );
		}
		
		public void newLine( int w, int h ) {
			
			height += lineHeight;
			if (width < lineWidth) {
				width = lineWidth;
			}
			
			lineWidth = w;
			lineHeight = h;
			
			x = 0;
			y = height;
		}
	}
}
