package com.watabou.newquay;

import java.util.ArrayList;

import com.watabou.newquay.utils.Point;
import com.watabou.newquay.utils.PointF;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.PixelFormat;
import android.graphics.PorterDuff;

public class Camera extends Gizmo {

	protected static ArrayList<Camera> all = new ArrayList<Camera>();
	
	public static Camera main;
	
	public float zoom;
	
	public int x;
	public int y;
	public int width;
	public int height;
	
	public PointF scroll;
	public Visual target;
	
	public Bitmap bitmap;
	public Canvas canvas;
	
	protected boolean clearOnUpdate = true;
	protected int bgColor = 0x00000000;
	
	public static Camera reset() {
		return reset( createFullscreen( 1 ) );
	}
	
	public static Camera reset( Camera newCamera ) {
		int length = all.size();
		for (int i=0; i < length; i++) {
			all.get( i ).destroy();
		}
		all.clear();
		
		return main = add( newCamera );
	}
	
	public static Camera add( Camera camera ) {
		all.add( camera );
		return camera;
	}
	
	public static Camera remove( Camera camera ) {
		all.remove( camera );
		return camera;
	}
	
	public static void updateAll() {
		int length = all.size();
		for (int i=0; i < length; i++) {
			Camera c = all.get( i );
			if (c.exists && c.active) {
				c.update();
			}
		}
	}
	
	public static void drawAll( Canvas canvas ) {
		
		int length = all.size();
		for (int i=0; i < length; i++) {
			Camera c = all.get( i );
			if (c.exists && c.visible) {
				canvas.save();
				canvas.translate( c.x, c.y );
				canvas.scale( c.zoom, c.zoom );
				canvas.drawBitmap( c.bitmap, 0, 0, null );
				canvas.restore();
			}
		}
	}

	public static Camera createFullscreen( float zoom ) {
		return createFullscreen( zoom, false );
	}
	
	//      
	public static Camera createFullscreen( float zoom, boolean transparent ) {
		int w = (int)Math.ceil( Game.width / zoom );
		int h = (int)Math.ceil( Game.height / zoom );
		return new Camera( 
			(int)(Game.width - w * zoom) / 2, 
			(int)(Game.height - h * zoom) / 2, 
			w, h, zoom, transparent );
	}
	
	//   ,   ,  transparent = false
	public Camera( int x, int y, int width, int height, float zoom ) {
		this( x, y, width, height, zoom, false );
	}
	
	public Camera( int x, int y, int width, int height, float zoom, boolean transparent ) {
		
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.zoom = zoom;
		
		scroll = new PointF();
		
		//  PixelFormat  Bitmap.Config   2  - RGB_565  ARGB_8888.
		//         - RGB_565,   
		//     (   ),  
		//   ARGB_8888   .
		bitmap = Bitmap.createBitmap( width, height, 
			Game.pixelFormat == PixelFormat.RGB_565 && !transparent ? 
				Bitmap.Config.RGB_565 : 
				Bitmap.Config.ARGB_8888 );
		canvas = new Canvas( bitmap );
	}
	
	//      
	//     (canvas)
	public Camera( Canvas canvas ) {
		setBuffer( null, canvas );
	}
	
	//      
	//     (bitmap)
	public Camera( Bitmap bitmap ) {
		setBuffer( bitmap, canvas );
	}
	
	//        
	private void setBuffer( Bitmap bitmap, Canvas canvas ) {
		
		if (bitmap == null) {
			this.bitmap = null;
			this.canvas = canvas;
		} else {
			this.bitmap = bitmap;
			this.canvas = new Canvas( bitmap );
		}
		
		this.x = 0;
		this.y = 0;
		this.width = this.canvas.getWidth();
		this.height = this.canvas.getHeight();
		this.zoom = 1;
		
		scroll = new PointF();
	}
	
	@Override
	public void destroy() {
		if (bitmap != null) {
			bitmap.recycle();
		}
		bitmap = null;
		canvas = null;
	}
	
	@Override
	public void update() {
		super.update();
		
		if (clearOnUpdate) {
			canvas.drawColor( bgColor, PorterDuff.Mode.SRC );
		}
		
		if (target != null) {
			focusOn( target );
		}
	}
	
	//   
	public void bgColor( int value ) {
		clearOnUpdate = true;
		bgColor = value;
	}
	
	//    ,   
	//  , ..     
	public void setTransparent() {
		clearOnUpdate = true;
		bgColor = 0x00000000;
	}
	
	//   clearOnUpdate,   
	//   update   
	public void noBgColor() {
		clearOnUpdate = false;
	}
	
	public PointF center() {
		return new PointF( width / 2, height / 2 );
	}
	
	//  scroll  ,    
	//      x, y
	public void focusOn( float x, float y ) {
		scroll.set( x - width / 2, y - height / 2 );
	}
	
	//  scroll  ,    
	//      p
	public void focusOn( PointF point ) {
		focusOn( point.x, point.y );
	}
	
	//  scroll  ,    
	//     visual
	public void focusOn( Visual visual ) {
		focusOn( visual.center() );
	}
	
	//         
	public PointF screenToCamera( int x, int y ) {
		return new PointF( 
			(x - this.x) / zoom + scroll.x, 
			(y - this.y) / zoom + scroll.y );
	}
	
	public Point cameraToScreen( float x, float y ) {
		return new Point(
			(int)((x - scroll.x) * zoom + this.x),
			(int)((y - scroll.y) * zoom + this.y));
	}
	
	public float screenWidth() {
		return width * zoom; 
	}
	
	public float screenHeight() {
		return height * zoom; 
	}
}
