package com.watabou.newquay;

import java.util.ArrayList;

import com.watabou.newquay.utils.BitmapCache;
import com.watabou.newquay.utils.SystemTime;

import android.app.Activity;
import android.graphics.Canvas;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnTouchListener;

public class Game extends Activity implements Callback, OnTouchListener {

	//    , ,  ,    Flixel
	//          
	
	public static Game instance;
	
	//    ,     ,
	//   1,       
	public static int width;
	public static int height;
	public static int pixelFormat;
	
	// : mdpi=1, hdpi=1.5, xhdpi=2
	public static float density = 1;
	
	//   
	public Scene scene;
	//  ,    
	protected Scene requestedScene;
	//  ,        sceneClass
	protected boolean requestedReset = true;
	//   
	protected Class<? extends Scene> sceneClass;
	
	//   (      )
	//     , , ,    
	protected long now;
	// ,       
	//     ,   now
	protected long step;
	
	public static float timeScale = 1f;
	public static float elapsed = 0f;
	
	// Obviously ,    ,     
	//    
	protected SurfaceView view;
	protected SurfaceHolder holder;
	protected GameThread thread;
	
	//  ,    "",     
	// -            -
	// onDestroy  surfaceDestroyed,     , 
	//        destroyGame
	protected boolean finished = false;
	
	// ,     ,      
	//     (     )
	protected ArrayList<MotionEvent> motionEvents = new ArrayList<MotionEvent>();
	
	// ,     ,      
	//     (     )
	protected ArrayList<KeyEvent> keysEvents = new ArrayList<KeyEvent>();
	
	// ,        
	public Game( Class<? extends Scene> c ) {
		super();
		sceneClass = c;

		BitmapCache.context = instance = this;
	}
	
	@Override
	public void onCreate( Bundle savedInstanceState ) {
		
		super.onCreate( savedInstanceState );
		
		DisplayMetrics m = new DisplayMetrics();
		getWindowManager().getDefaultDisplay().getMetrics( m );
		density = m.density;
		
		view = new SurfaceView( this );
		view.setOnTouchListener( this );
		setContentView( view );
		
		holder = view.getHolder();
		holder.addCallback( this );
	}
	
	@Override
	public void onResume() {
		super.onResume();
	}
	
	@Override
	public void onPause() {
		super.onPause();
	}
	
	@Override
	//  thread == null,    surfaceDestroyed   ,
	//    destroyGame
	public void onDestroy() {

		super.onDestroy();
		
		finished = true;
		if (thread == null) {
			destroyGame();
		}
	}
	
	@Override
	//   ,       Touchscreen-
	public boolean onTouch( View view, MotionEvent event ) {
		synchronized (motionEvents) {
			motionEvents.add( MotionEvent.obtain( event ) );
		}
		return true;
	}
	
	@Override
	public boolean onKeyDown( int keyCode, KeyEvent event ) {
		synchronized (motionEvents) {
			keysEvents.add( event );
		}
		return true;
	}
	
	@Override
	public boolean onKeyUp( int keyCode, KeyEvent event ) {
		synchronized (motionEvents) {
			keysEvents.add( event );
		}
		return true;
	}

	@Override
	public void surfaceChanged( SurfaceHolder holder, int format, int width, int height ) {
		Game.width = width;
		Game.height = height;
		Game.pixelFormat = format;
	}

	@Override
	public void surfaceCreated( SurfaceHolder holder ) {
		thread = new GameThread();
		thread.running( true );
		thread.start();
	}

	@Override
	//       thread = null, 
	// ,    
	public void surfaceDestroyed( SurfaceHolder holder ) {
	
		boolean retry = true;
		thread.running( false );
		while (retry) {
			try {
				thread.join();
				retry = false;
			} catch (InterruptedException e) {
            }
		}
		
		thread = null;
	}
	
	//    (  )
	//     , ,   
	protected void destroyGame() {
		if (scene != null) {
			scene.destroy();
			scene = null;
		}
		
		instance = null;
	}
	
	//     
	public static void resetScene() {
		switchScene( instance.sceneClass );
	}
	
	//      
	public static void switchScene( Class<? extends Scene> c ) {
		instance.sceneClass = c;
		instance.requestedReset = true;
	}
	
	protected void step() {
		
		if (requestedReset) {
			requestedReset = false;
			try {
				requestedScene = sceneClass.newInstance();
				switchScene();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		update();
	}
	
	//     ,   ,
	//        
	protected void switchScene() {

		Camera.reset();
		
		if (scene != null) {
			scene.destroy();
		}
		scene = requestedScene;
		scene.create();
		
		Game.elapsed = 0f;
		Game.timeScale = 1f;
	}
	
	//          (, ,
	//    ,    ),   
	// ,     
	protected void update() {
		Game.elapsed = (float)(Game.timeScale * step * 0.000000001);
		
		synchronized (motionEvents) {
			Touchscreen.processTouchEvents( motionEvents );
			motionEvents.clear();
		}
		synchronized (keysEvents) {
			Keys.processTouchEvents( keysEvents );
			keysEvents.clear();
		}
		
		scene.update();		
		Camera.updateAll();
	}
	
	//   :       
	//  ,        
	protected void draw( Canvas canvas ) {
		scene.draw();
		Camera.drawAll( canvas );
	}
	
	private class GameThread extends Thread {
		
		private boolean running = false;
		
		@Override
		public void run() {
			
			while (running) {
				
				SystemTime.tick();
				long rightNow = SystemTime.now;
				step = (now == 0 ? 0 : rightNow - now);
				now = rightNow;
				
				step();
				
				Canvas canvas = null;
				try {
					canvas = holder.lockCanvas();
					if (canvas != null) {
						//  ,      
						//  ,    ,
						//      
						draw( canvas );
					}
				} finally {
					if (canvas != null) {
						holder.unlockCanvasAndPost( canvas );
					}
				}
			}
			
			if (finished) {
				destroyGame();
			}
		}
		
		public void running( boolean value ) {
			running = value;
		}
	}
}
