package com.watabou.newquay;

import java.util.ArrayList;

public class Group extends Gizmo {

	//    ,   
	//       
	protected ArrayList<Gizmo> members;
	
	//      
	// ,         
	public int length;
	
	public Group() {
		members = new ArrayList<Gizmo>();
		length = 0;
	}
	
	@Override
	public void destroy() {
		//    
		//    (   )
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null) {
				g.destroy();
			}
		}
		
		members.clear();
		members = null;
		length = 0;
	}
	
	@Override
	public void update() {
		//     
		//    
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && g.exists && g.active) {
				g.update();
			}
		}
	}
	
	@Override
	public void draw() {
		//     
		//    
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && g.exists && g.visible) {
				g.draw();
			}
		}
	}
	
	@Override
	public void kill() {
		//     
		//   
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && g.exists) {
				g.kill();
			}
		}
		
		super.kill();
	}
	
	//    
	public int indexOf( Gizmo g ) {
		return members.indexOf( g );
	}
	
	public Gizmo add( Gizmo g ) {
		
		//       
	/*	if (members.indexOf( g ) != -1) {
			return g;
		}*/
		if (g.parent == this) {
			return g;
		}
		
		//     , 
		//    
		if (g.parent != null) {
			//       
			//  ,     
			g.parent.remove( g );
		}
		
		//     " " (null),
		//      ...
		for (int i=0; i < length; i++) {
			if (members.get( i ) == null) {
				members.set( i, g );
				g.parent = this;
				return g;
			}
		}
		
		//      
		members.add( g );
		g.parent = this;
		length++;
		return g;
	}
	
	public Gizmo recycle( Class<? extends Gizmo> c ) {

		//      
		//    
		Gizmo g = getFirstAvailable( c );
		if (g != null) {
			
			//  ,    
			return g;
			
		} else if (c == null) {
			
			//       ,   
			return null;
			
		} else {
			
			//       
			// ,      
			try {
				return add( c.newInstance() );
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		return null;
	}
	
	//    ,  
	//      -  
	//   
	public Gizmo erase( Gizmo g ) {
		int index = members.indexOf( g );
		if (index != -1) {
			members.set( index, null );
			g.parent = null;
			return g;
		} else {
			return null;
		}
	}
	
	// ""    
	public Gizmo remove( Gizmo g ) {
		if (members.remove( g )) {
			length--;
			g.parent = null;
			return g;
		} else {
			return null;
		}
	}
	
	//      ,  
	//    ,    
	public Gizmo replace( Gizmo oldOne, Gizmo newOne ) {
		int index = members.indexOf( oldOne );
		if (index != -1) {
			members.set( index, newOne );
			newOne.parent = this;
			oldOne.parent = null;
			return newOne;
		} else {
			return null;
		}
	}
	
	//       
	//     -     null,  
	// " "    ,    
	//  
	public Gizmo getFirstAvailable( Class<? extends Gizmo> c ) {
		
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && !g.exists && ((c == null) || c.isInstance( g ))) {
				return g;
			}
		}
		
		return null;
	}
	
	//      
	public int countLiving() {
		
		int count = 0;
		
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && g.exists && g.alive) {
				count++;
			}
		}
		
		return count;
	}
	
	//     
	public int countDead() {
		
		int count = 0;
		
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null && !g.alive) {
				count++;
			}
		}
		
		return count;
	}
	
	//     ,  
	//  ""   
	public Gizmo random() {
		if (length > 0) {
			return members.get( (int)(Math.random() * length) );
		} else {
			return null;
		}
	}
	
	//     
	public void clear() {
		for (int i=0; i < length; i++) {
			Gizmo g = members.get( i );
			if (g != null) {
				g.parent = null;
			}
		}
		members.clear();
		length = 0;
	}
	
	public Gizmo bringToFront( Gizmo g ) {
		if (members.contains( g )) {
			members.remove( g );
			members.add( g );
			return g;
		} else {
			return null;
		}
	}
	
	public Gizmo sendToBack( Gizmo g ) {
		if (members.contains( g )) {
			members.remove( g );
			members.add( 0, g );
			return g;
		} else {
			return null;
		}
	}
}
