package com.watabou.newquay;

import com.watabou.newquay.utils.BitmapCache;
import com.watabou.newquay.utils.PointF;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;

public class Image extends Visual {

	public PointF scale;
	public PointF origin;
	
	protected Bitmap bitmap;
	protected Rect frame;
	
	private Rect dstRect = new Rect();
	
	protected Paint paint;
	
	public Image() {
		super( 0, 0, 0, 0 );
		
		scale = new PointF( 1, 1 );
		origin = new PointF( 0, 0 );
	}
	
	public Image( Bitmap bmp ) {
		this();
		bitmap( bmp );
	}
	
	public Image( String assetName ) {
		this( BitmapCache.get( assetName ) );
	}
	
	public Image( int resID ) {
		this( BitmapCache.get( resID ) );
	}
	
	@Override
	public void draw() {
		
		super.draw();
		
		Camera c = camera();
		
		float x = this.x - c.scroll.x * parallax;
		float y = this.y - c.scroll.y * parallax;
		
		Canvas canvas = c.canvas;
		
		if (angle == 0 && scale.x == 1 && scale.y == 1) {
			
			dstRect.offsetTo( (int)x, (int)y );
			canvas.drawBitmap( bitmap, frame, dstRect, paint );
			
		} else {

			canvas.save();
			
			canvas.translate( x + origin.x, y + origin.y );
			canvas.scale( scale.x, scale.y );
			canvas.rotate( angle );
			canvas.translate( -origin.x, -origin.y );
			
			dstRect.offsetTo( 0, 0 );
			canvas.drawBitmap( bitmap, frame, dstRect, paint );
			
			canvas.restore();
		}
	}
	
	public void frame( Rect rect ) {
		
		if (frame != null) {
			frame.set( rect );
		} else {
			frame = rect;
		}
		dstRect.set( rect );
		
		width = rect.width();
		height = rect.height();
		
		origin.set( width / 2, height / 2 );
	}
	
	public void filter( boolean value ) {
		paint = value ? new Paint( Paint.FILTER_BITMAP_FLAG ) : null;
	}
	
	public float alpha() {
		return paint != null ? paint.getAlpha() / 255f : 1;
	}
	
	public void alpha( float value ) {
		if (value < 1) {
			paint = new Paint();
			paint.setAlpha( (int)(255 * value) );
		} else {
			paint = null;
		}
		
	}
	
	@Override
	public boolean overlapsPoint( float x, float y ) {
		return x >= this.x && x < this.x + width * scale.x && y >= this.y && y < this.y + height * scale.y ;
	}
	
	public Bitmap bitmap() {
		return bitmap;
	}
	
	public void bitmap( Bitmap value ) {
		bitmap = value;
		frame( new Rect( 0, 0, bitmap.getWidth(), bitmap.getHeight() ) );
	}
	
	public void bitmap( int width, int height ) {
		bitmap( Bitmap.createBitmap( width, height, Bitmap.Config.ARGB_8888 ) );
	}
	
	// :        
	public void recycle() {
		bitmap.recycle();
	}
	
	public static Image create( int width, int height, int color ) {
		Image img = new Image( Bitmap.createBitmap( width, height, Bitmap.Config.ARGB_8888 ) );
		img.bitmap.eraseColor( color );
		return img;
	}
}
