package com.watabou.newquay;

import com.watabou.newquay.utils.Film;

import android.graphics.Bitmap;
import android.graphics.Rect;

public class MovieClip extends Image {

	protected Animation curAnim;
	
	protected int curFrame;
	protected float frameTimer;
	protected boolean finished;
	
	public Listener listener;
	
	public MovieClip() {
		super();
	}
	
	public MovieClip( Bitmap bmp ) {
		super( bmp );
	}
	
	@Override
	public void update() {
		super.update();
		updateAnimation();
	}
	
	protected void updateAnimation() {
		if (curAnim != null && curAnim.delay > 0 && (curAnim.looped || !finished)) {
			
			int lastFrame = curFrame;
			
			frameTimer += Game.elapsed;
			while (frameTimer > curAnim.delay) {
				frameTimer -= curAnim.delay;
				if (curFrame == curAnim.frames.length - 1) {
					if (curAnim.looped) {
						curFrame = 0;
					}
					if (listener != null) {
						listener.onComplete( curAnim );
					}
					finished = true;
				} else {
					curFrame++;
				}
			}
			
			if (curFrame != lastFrame) {
				frame( curAnim.frames[curFrame] );
			}
			
		}
	}
	
	public void play( Animation anim ) {
		play( anim, false );
	}

	public void play( Animation anim, boolean force ) {
		if (!force && (curAnim != null) && (curAnim == anim) && (curAnim.looped || !finished)) {
			return;
		}
		
		curAnim = anim;
		curFrame = 0;
		finished = false;
		
		frameTimer = 0;
		
		frame( anim.frames[curFrame] );
	}
	
	public static class Animation {
		
		public float delay;
		public Rect[] frames;
		public boolean looped;
		
		public Animation( int fps, boolean looped ) {
			this.delay = 1f / fps;
			this.looped = looped;
		}
		
		public Animation frames( Rect... frames ) {
			this.frames = frames;
			return this;
		}
		
		public Animation frames( Film film, Object... frames ) {
			this.frames = new Rect[frames.length];
			for (int i=0; i < frames.length; i++) {
				this.frames[i] = film.get( frames[i] );
			}
			return this;
		}
	}
	
	public interface Listener {
		void onComplete( Animation anim );
	}
}
