package com.watabou.newquay;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.RectF;

import com.watabou.newquay.utils.BitmapCache;

public class NinePatch extends Visual {
	
	protected Bitmap bitmap;
	protected int marginLeft;
	protected int marginRight;
	protected int marginTop;
	protected int marginBottom;
	
	protected Paint paint;
	
	private Rect src0;
	private Rect src1;
	private Rect src2;
	private Rect src3;
	private Rect src4;
	private Rect src5;
	private Rect src6;
	private Rect src7;
	private Rect src8;
	private RectF dst;
	
	public NinePatch( Bitmap bitmap, int left, int top, int right, int bottom ) {
		super( 0, 0, bitmap.getWidth(), bitmap.getHeight() );
		
		this.bitmap = bitmap;
		this.marginLeft		= left;
		this.marginRight	= right;
		this.marginTop		= top;
		this.marginBottom	= bottom;

		int w = (int)width;
		int h = (int)height;
		
		src0 = new Rect( 0, 0, left, top );
		src1 = new Rect( left, 0, w - right, top );
		src2 = new Rect( w - right, 0, w, top );
		
		src3 = new Rect( 0, top, left, h - bottom );
		src4 = new Rect( left, top, w - right, h - bottom );
		src5 = new Rect( w - right, top, w, h - bottom );
		
		src6 = new Rect( 0, h - bottom, left, h );
		src7 = new Rect( left, h - bottom, w - right, h );
		src8 = new Rect( w - right, h - bottom, w, h );
		
		dst = new RectF();
	}
	
	public NinePatch( String assetName, int left, int top, int right, int bottom ) {
		this( BitmapCache.get( assetName ), left, top, right, bottom );
	}
	
	public NinePatch( int resID, int left, int top, int right, int bottom ) {
		this( BitmapCache.get( resID ), left, top, right, bottom );
	}
	
	@Override
	public void draw() {
		
		super.draw();
		
		Camera c = camera();
		
		float x = this.x - c.scroll.x * parallax;
		float y = this.y - c.scroll.y * parallax;
		
		Canvas canvas = c.canvas;
		
		canvas.save();
		canvas.translate( x, y );
		canvas.rotate( angle );
		
		float right = width - marginRight;
		float bottom = height - marginBottom;

		dst.set( src0 );
		canvas.drawBitmap( bitmap, src0, dst, paint );
		
		dst.set( src1 );
		dst.right = right;
		canvas.drawBitmap( bitmap, src1, dst, paint );
		
		dst.set( src2 );
		dst.offsetTo( right, 0 );
		canvas.drawBitmap( bitmap, src2, dst, paint );
		
		dst.set( src3 );
		dst.bottom = bottom;
		canvas.drawBitmap( bitmap, src3, dst, paint );
		
		dst.set( src4 );
		dst.right = right;
		dst.bottom = bottom;
		canvas.drawBitmap( bitmap, src4, dst, paint );
		
		dst.set( src5 );
		dst.offsetTo( right, marginTop );
		dst.bottom = bottom;
		canvas.drawBitmap( bitmap, src5, dst, paint );
		
		dst.set( src6 );
		dst.offsetTo( 0, bottom );
		canvas.drawBitmap( bitmap, src6, dst, paint );
		
		dst.set( src7 );
		dst.offsetTo( marginLeft, bottom );
		dst.right = right;
		canvas.drawBitmap( bitmap, src7, dst, paint );
		
		dst.set( src8 );
		dst.offsetTo( right, bottom );
		canvas.drawBitmap( bitmap, src8, dst, paint );
		
		canvas.restore();
	}
	
	public int marginLeft() {
		return marginLeft;
	}
	
	public int marginRight() {
		return marginRight;
	}
	
	public int marginTop() {
		return marginTop;
	}
	
	public int marginBottom() {
		return marginBottom;
	}
	
	public void filter( boolean value ) {
		paint = value ? new Paint( Paint.FILTER_BITMAP_FLAG ) : null;
	}
	
	public float alpha() {
		return paint != null ? paint.getAlpha() / 255f : 1;
	}
	
	public void alpha( float value ) {
		if (value < 1) {
			paint = new Paint();
			paint.setAlpha( (int)(255 * value) );
		} else {
			paint = null;
		}
		
	}
}
