package com.watabou.newquay;

import com.watabou.newquay.utils.Signal;

import android.graphics.PointF;

abstract public class PanControl extends Gizmo implements Signal.Listener<Touchscreen.Touch> {

	public float startX;
	public float startY;
	
	public float x;
	public float y;
	
	public PointF thisOffset = new PointF();
	public PointF fullOffset = new PointF();
	
	public boolean started = false;
	public boolean dragged = false;
		
	public PanControl() {
		super();
		Touchscreen.event.add( this );
	}
	
	@Override
	public void destroy() {
		Touchscreen.event.remove( this );
	}
		
	@Override
	public void onSignal( Touchscreen.Touch touch ) {

		if (touch != null && touch.down) {
			
			startX = x = Touchscreen.x;
			startY = y = Touchscreen.y;
			
			started = start();
			dragged = false;
			if (started) {			
				thisOffset.set( 0, 0 );
				fullOffset.set( 0, 0 );
			}
			
		} else if (started) {
			
			if (!Touchscreen.touched) {

				started = false;
				end();
				
			} else {
				
				float dx = Touchscreen.x - x;
				float dy = Touchscreen.y - y;
				
				thisOffset.set( dx, dy );
				fullOffset.offset( dx, dy );
				
				x = Touchscreen.x;
				y = Touchscreen.y;
				
				if (dragged || fullOffset.length() > 10) {
					drag();	
					dragged = true;
				}
			}
			
			if (dragged) {
				Touchscreen.event.cancel();
			}
		}

	}
		
	abstract public boolean start();
	abstract public void drag();
	abstract public void end();

	public static class VisualDragger extends PanControl {

		public Visual visual;
		public Camera camera;
		
		public VisualDragger( Visual visual ) {
			this.visual = visual;
			this.camera = visual.camera != null ? visual.camera : Camera.main;
		}
		
		@Override
		public boolean start() {
			return visual.overlapsScreenPoint( (int)Touchscreen.x, (int)Touchscreen.y );
		}

		@Override
		public void drag() {
			visual.x += thisOffset.x / camera.zoom;
			visual.y += thisOffset.y / camera.zoom;
		}

		@Override
		public void end() {
		}
		
	}
		
	public static class CameraDragger extends PanControl {

		public Camera camera;
		
		public CameraDragger() {
			this( Camera.main );
		}
		
		public CameraDragger( Camera camera ) {
			this.camera = camera;
		}
		
		@Override
		public boolean start() {
			return true;
		}

		@Override
		public void drag() {
			camera.scroll.offset( 
				-thisOffset.x / camera.zoom, 
				-thisOffset.y / camera.zoom );
		}

		@Override
		public void end() {
		}
		
	}
	
}

