package com.watabou.newquay;

import android.graphics.Bitmap;
import android.graphics.BitmapShader;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Shader;

public class SkinnedBlock extends Visual {

	protected Bitmap bitmap;
	protected Matrix matrix;
	protected Paint paint;
	protected BitmapShader shader;
	
	protected float scaleX;
	protected float scaleY;
	
	protected float offsetX;
	protected float offsetY;
	
	public SkinnedBlock( float x, float y, float width, float height, Bitmap bitmap ) {
		super( x, y, width, height );
		
		this.bitmap = bitmap;
		
		matrix = new Matrix();
		shader = new BitmapShader( bitmap, Shader.TileMode.REPEAT, Shader.TileMode.REPEAT );
		paint = new Paint();
		paint.setShader( shader );
		
		scaleX = 1;
		scaleY = 1;
		
		offsetX = 0;
		offsetY = 0;
		
		updateMatrix();
	}
	
	@Override
	public void draw() {
		
		super.draw();
		
		Camera c = camera();
		
		float x = this.x - c.scroll.x * parallax;
		float y = this.y - c.scroll.y * parallax;
		
		c.canvas.drawRect( x, y, x + width, y + height, paint );

	}
	
	public void offsetTo( float x, float y ) {
		offsetX = x;
		offsetY = y;
		updateMatrix();
	}
	
	public void offset( float x, float y ) {
		offsetX += x;
		offsetY += y;
		updateMatrix();
	}
	
	public void scale( float x, float y ) {
		scaleX = x;
		scaleY = y;
		updateMatrix();
	}
	
	protected void updateMatrix() {
		matrix.setScale( scaleX, scaleY );
		matrix.postTranslate( offsetX, offsetY );
		shader.setLocalMatrix( matrix );
	}
}
