package com.watabou.newquay;

import com.watabou.newquay.utils.Film;

import android.graphics.Canvas;
import android.graphics.Rect;

public class Tilemap extends Visual {

	protected Film tileset;
	
	protected int[] data;
	protected int mapWidth;
	protected int mapHeight;
	
	private Rect dstRect;
	private int cellW;
	private int cellH;
	
	public Tilemap( Film tileset ) {
		super( 0, 0, 0, 0 );
		
		this.tileset = tileset;
		dstRect = new Rect( tileset.get( 0 ) );
		cellW = dstRect.width();
		cellH = dstRect.height();
	}
	
	public void map( int[] data, int cols ) {
		this.data = data;
		mapWidth = cols;
		mapHeight = data.length / cols;
		
		width = cellW * mapWidth;
		height = cellH * mapHeight;
	}
	
	@Override
	public void draw() {

		super.draw();
		
		Camera c = camera();
		
		int x = (int)(this.x - c.scroll.x * parallax);
		int y = (int)(this.y - c.scroll.y * parallax);
		
		Canvas canvas = c.canvas;
		
		int left = x < 0 ? -x / cellW : 0;
		int top = y < 0 ? -y / cellH : 0;
		
		int right = left + c.width / cellW + 1;
		if (right >= mapWidth) {
			right = mapWidth - 1;
		}
		
		int bottom = top + c.height / cellH + 1;
		if (bottom >= mapHeight) {
			bottom = mapHeight - 1;
		}
		
		int cellY = y + cellH * top;
		for (int i=top; i <= bottom; i++) {
			int cellX = x + cellW * left;
			int pos = i * mapWidth + left;
			for (int j=left; j <= right; j++) {
				Rect srcRect = tileset.get( data[pos++] );
				if (srcRect != null) {
					dstRect.offsetTo( cellX, cellY );
					canvas.drawBitmap( tileset.bitmap, srcRect, dstRect, null );
				}
				cellX += cellW;
			}
			cellY += cellH;
		}

	}
}
