package com.watabou.newquay.utils;

import java.util.Arrays;
import java.util.LinkedList;

public class PathFinder {

	private static int width;
	private static int height;
	
	private static int[] distance = new int[0];
	private static int[] queue;
	
	public static final Point WW = new Point( -1,  0 );
	public static final Point EE = new Point( +1,  0 );
	public static final Point NN = new Point(  0, -1 );
	public static final Point SS = new Point(  0, +1 );
	public static final Point NW = new Point( -1, -1 );
	public static final Point SW = new Point( -1, +1 );
	public static final Point SE = new Point( +1, +1 );
	public static final Point NE = new Point( +1, -1 );
	
	public static final Point[] DIR = {
		WW, EE, NN, SS,
		NW, SW, SE, NE
	};
	
	public static void setMapSize( int width, int height ) {
		PathFinder.width	= width;
		PathFinder.height	= height;
		int size = width * height;
		if (distance.length != size) {
			distance = new int[size];
			queue = new int[size];
		}
	}
	
	public static Path find( Point from, Point to, boolean[] passable ) {

		if (!buildDistanceMap( from, to, passable )) {
			return null;
		}
		
		Path result = new Path();
		int sx = from.x;
		int sy = from.y;
		//      "  ",   
		//          
		//   ,      (  0)
		do {
			int minD = distance[sy * width + sx];
			int minx = sx;
			int miny = sy;
			
			for (int i=0; i < DIR.length; i++) {
				
				Point dir = DIR[i];
				int nx = sx + dir.x;
				int ny = sy + dir.y;
				
				if (nx >= 0 && nx < width && ny >= 0 && ny < height) {
					int thisD = distance[ny * width + nx];
					if (thisD < minD) {
						minD = thisD;
						minx = nx;
						miny = ny;
					}
				}
			}
			sx = minx;
			sy = miny;
			result.add( new Point( sx, sy ) );
		} while (sx != to.x || sy != to.y);
		
		return result;
	}
	
	public static Point getStep( Point from, Point to, boolean[] passable ) {
		
		if (!buildDistanceMap( from, to, passable )) {
			return null;
		}
		
		int sx = from.x;
		int sy = from.y;
		//      "  ", 
		//        
		int minD = distance[sy * width + sx];
		int minx = sx;
		int miny = sy;
		
		for (int i=0; i < DIR.length; i++) {
			
			Point dir = DIR[i];
			int nx = sx + dir.x;
			int ny = sy + dir.y;
			
			if (nx >= 0 && nx < width && ny >= 0 && ny < height) {
				int thisD = distance[ny * width + nx];
				if (thisD < minD) {
					minD = thisD;
					minx = nx;
					miny = ny;
				}
			}
		}

		return new Point( minx, miny );
	}
	
	private static boolean buildDistanceMap( Point from, Point to, boolean[] passable ) {
		
		int fromPos = from.y * width + from.x;
		int toPos = to.y * width + to.x;
		if (fromPos == toPos) {
			return false;
		}
		
		Arrays.fill( distance, Integer.MAX_VALUE );
		
		boolean pathFound = false;
		
		int head = 0;
		int tail = 0;
		// Add to queue
			queue[tail++] = toPos;
			distance[toPos] = 0;
		while (head < tail) {
			
			// Remove from queue
				int step = queue[head++];
			if (step == fromPos) {
				pathFound = true;
				break;
			}
			int nextDistance = distance[step] + 1;
			
			int sx = step % width;
			int sy = step / width;
			
			for (int i=0; i < DIR.length; i++) {

				Point dir = DIR[i];
				int stepX = sx + dir.x;
				int stepY = sy + dir.y;

				if (stepX >= 0 && stepX < width && stepY >= 0 && stepY < height) {
					int stepPos = stepY * width + stepX;
					if (stepPos == fromPos || (passable[stepPos] && (distance[stepPos] > nextDistance))) {
						// Add to queue
							queue[tail++] = stepPos;
							distance[stepPos] = nextDistance;
					}
				}
					
			}
		}
		
		return pathFound;
	}
	
	@SuppressWarnings("serial")
	public static class Path extends LinkedList<Point> {
	}
}
