package com.watabou.newquay.utils;

public class Rect {

	public int left;
	public int top;
	public int right;
	public int bottom;
	
	public Rect() {
		this( 0, 0, 0, 0 );
	}
	
	public Rect( Rect rect ) {
		this( rect.left, rect.top, rect.right, rect.bottom );
	}
	
	public Rect( int left, int top, int right, int bottom ) {
		this.left	= left;
		this.top	= top;
		this.right	= right;
		this.bottom	= bottom;
	}
	
	public int width() {
		return right - left;
	}
	
	public int height() {
		return bottom - top;
	}
	
	public int square() {
		return (right - left) * (bottom - top);
	}
	
	public boolean isEmpty() {
		return right <= left || bottom <= top;
	}
	
	public Rect set( Rect rect ) {
		left	= rect.left;
		top		= rect.top;
		right	= rect.right;
		bottom	= rect.bottom;
		return this;
	}
	
	public Rect intersect( Rect other ) {
		Rect result = new Rect();
		result.left		= Math.max( left, other.left );
		result.right	= Math.min( right, other.right );
		result.top		= Math.max( top, other.top );
		result.bottom	= Math.min( bottom, other.bottom );
		return result;
	}
	
	public boolean inside( Point p ) {
		return p.x >= left && p.x < right && p.y >= top && p.y < bottom;
	}
	
	public Rect shrink( int d ) {
		return new Rect( left + d, top + d, right - d, bottom - d );
	}
	
	public Rect shrink() {
		return shrink( 1 );
	}
	
}
